/*
 * Hookup (Nano):
 *   BUSY D7 (INPUT)
 *   RST  D8 (OUTPUT)
 *   DC   D9 (OUTPUT)
 *   CS   D10 (OUTPUT)
 *   
 *   SPI (2 MHz):
 *     CS   D10 (SS)
 *     DIN  D11 (MOSI)
 *     -    D12 (MISO)
 *     CLK  D13 (SCLK)
 *     
 *  Display resolution
 *    EPD_WIDTH      128
 *    EPD_HEIGHT     296
 *     
 *  Class Edp:
 *    unsigned long width;
 *    unsigned long height;
 *    int  Init(const unsigned char* lut);
 *    void SendCommand(unsigned char command);
 *    void SendData(unsigned char data);
 *    void WaitUntilIdle(void);
 *    void Reset(void);
 *    void SetFrameMemory(
 *        const unsigned char* image_buffer,
 *        int x,
 *        int y,
 *        int image_width,
 *        int image_height
 *    );
 *    void SetFrameMemory(const unsigned char* image_buffer);
 *    void ClearFrameMemory(unsigned char color);
 *    void DisplayFrame(void);
 *    void Sleep(void);
 * 
 *     
 *  Display orientation
 *     ROTATE_0            0
 *     ROTATE_90           1
 *     ROTATE_180          2
 *     ROTATE_270          3
 *  Color inverse. 1 or 0 = set or reset a bit if set a colored pixel
 *    IF_INVERT_COLOR     1
 *
 *  class Paint:
 *    Paint(unsigned char* image, int width, int height);
 *    void Clear(int colored);
 *    int  GetWidth(void);
 *    void SetWidth(int width);
 *    int  GetHeight(void);
 *    void SetHeight(int height);
 *    int  GetRotate(void);
 *    void SetRotate(int rotate);
 *    unsigned char* GetImage(void);
 *    void DrawAbsolutePixel(int x, int y, int colored);
 *    void DrawPixel(int x, int y, int colored);
 *    void DrawCharAt(int x, int y, char ascii_char, sFONT* font, int colored);
 *    void DrawStringAt(int x, int y, const char* text, sFONT* font, int colored);
 *    void DrawLine(int x0, int y0, int x1, int y1, int colored);
 *    void DrawHorizontalLine(int x, int y, int width, int colored);
 *    void DrawVerticalLine(int x, int y, int height, int colored);
 *    void DrawRectangle(int x0, int y0, int x1, int y1, int colored);
 *    void DrawFilledRectangle(int x0, int y0, int x1, int y1, int colored);
 *    void DrawCircle(int x, int y, int radius, int colored);
 *    void DrawFilledCircle(int x, int y, int radius, int colored);
 * 
 */

#include <SPI.h>
#include <epd2in9.h>
#include <epdpaint.h>
#include "imagedata.h"

#define COLORED     0
#define UNCOLORED   1

/**
  * Due to RAM not enough in Arduino UNO, a frame buffer is not allowed.
  * In this case, a smaller image buffer is allocated and you have to 
  * update a partial display several times.
  * 1 byte = 8 pixels, therefore you have to set 8*N pixels at a time.
  */
unsigned char image[1024];
Paint paint(image, 0, 0);    // width should be the multiple of 8 
Epd epd;
unsigned long time_start_ms;
unsigned long time_now_s;

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  if (epd.Init(lut_full_update) != 0) {
      Serial.print("e-Paper init failed");
      return;
  }

  /** 
   *  there are 2 memory areas embedded in the e-paper display
   *  and once the display is refreshed, the memory area will be auto-toggled,
   *  i.e. the next action of SetFrameMemory will set the other memory area
   *  therefore you have to clear the frame memory twice.
   */
  epd.ClearFrameMemory(0xFF);   // bit set = white, bit reset = black
  epd.DisplayFrame();
  epd.ClearFrameMemory(0xFF);   // bit set = white, bit reset = black
  epd.DisplayFrame();

  paint.SetRotate(ROTATE_0);
  paint.SetWidth(128);
  paint.SetHeight(24);

  /* For simplicity, the arguments are explicit numerical coordinates */
  paint.Clear(COLORED);
  paint.DrawStringAt(0, 4, "Hello World!", &Font16, UNCOLORED);
  epd.SetFrameMemory(paint.GetImage(), 0, 4, paint.GetWidth(), paint.GetHeight());
  
  paint.Clear(UNCOLORED);
  paint.DrawStringAt(0, 4, "e-Paper Demo", &Font12, COLORED);
  epd.SetFrameMemory(paint.GetImage(), 16, 32, paint.GetWidth(), paint.GetHeight());

  paint.SetWidth(64);
  paint.SetHeight(64);
  
  paint.Clear(UNCOLORED);
  paint.DrawRectangle(0, 0, 40, 40, COLORED);
  paint.DrawLine(0, 0, 40, 40, COLORED);
  paint.DrawLine(40, 0, 0, 40, COLORED);
  epd.SetFrameMemory(paint.GetImage(), 8, 60, paint.GetWidth(), paint.GetHeight());

  paint.Clear(UNCOLORED);
  paint.DrawCircle(32, 32, 30, COLORED);
  epd.SetFrameMemory(paint.GetImage(), 60, 50, paint.GetWidth(), paint.GetHeight());

  paint.Clear(UNCOLORED);
  paint.DrawFilledRectangle(0, 0, 40, 40, COLORED);
  epd.SetFrameMemory(paint.GetImage(), 8, 130, paint.GetWidth(), paint.GetHeight());

  paint.Clear(UNCOLORED);
  paint.DrawFilledCircle(32, 32, 30, COLORED);
  epd.SetFrameMemory(paint.GetImage(), 60, 120, paint.GetWidth(), paint.GetHeight());
  epd.DisplayFrame();

  delay(10000);

  if (epd.Init(lut_partial_update) != 0) {
      Serial.print("e-Paper init failed");
      return;
  }

  /** 
   *  there are 2 memory areas embedded in the e-paper display
   *  and once the display is refreshed, the memory area will be auto-toggled,
   *  i.e. the next action of SetFrameMemory will set the other memory area
   *  therefore you have to set the frame memory and refresh the display twice.
   */
  epd.SetFrameMemory(IMAGE_DATA);
  epd.DisplayFrame();
  epd.SetFrameMemory(IMAGE_DATA);
  epd.DisplayFrame();

  time_start_ms = millis();
}

void loop() {
  // put your main code here, to run repeatedly:
  time_now_s = (millis() - time_start_ms) / 1000;
  char time_string[] = {'0', '0', ':', '0', '0', '\0'};
  time_string[0] = time_now_s / 60 / 10 + '0';
  time_string[1] = time_now_s / 60 % 10 + '0';
  time_string[3] = time_now_s % 60 / 10 + '0';
  time_string[4] = time_now_s % 60 % 10 + '0';

  paint.SetWidth(32);
  paint.SetHeight(96);
  paint.SetRotate(ROTATE_90);

  paint.Clear(UNCOLORED);
  paint.DrawStringAt(0, 4, time_string, &Font24, COLORED);
  epd.SetFrameMemory(paint.GetImage(), 96, 0, paint.GetWidth(), paint.GetHeight());
  epd.DisplayFrame();

  delay(1000);
}

